import os
import datetime
import redis
import requests
from cachecontrol import CacheControl
from cachecontrol.cache import DictCache
from cachecontrol.caches.redis_cache import RedisCache
from cachecontrol.heuristics import ExpiresAfter
from .logger import log


def print_response(r, *args, **kwargs):
    log.debug(
        "response",
        method=r.request.method,
        url=r.url,
        status=r.status_code,
        from_cache=r.from_cache,
    )


EXPIRATION_DELTA = {"minutes": 7}


class ExpiringRedisCache(RedisCache):
    """
    The built-in redis class has an expires arg, but no way to use it yet
    https://github.com/ionrock/cachecontrol/blob/master/cachecontrol/caches/redis_cache.py
    """

    def get(self, *args, **kwargs):
        try:
            return super().get(*args, **kwargs)
        except redis.ConnectionError as e:
            log.error("redis_connection_error", exc=e)

    def set(self, key, value):
        try:
            expires = datetime.datetime.utcnow() + datetime.timedelta(
                **EXPIRATION_DELTA
            )
            log.debug("redis_cache_set", key=key)
            return super().set(key, value, expires=expires)
        except redis.ConnectionError as e:
            log.error("redis_connection_error", exc=e)

    def delete(self, *args, **kwargs):
        try:
            return super().delete(*args, **kwargs)
        except redis.ConnectionError as e:
            log.error("redis_connection_error", exc=e)


# use an in-memory cache, and cache everything no matter what
# this way we can re-get urls across the code without hitting url again
requests_session = requests.session()
requests_session.headers.update({"User-Agent": "sitechecks"})
requests_session.hooks["response"].append(print_response)

cache = DictCache()

if "REDIS_URL" in os.environ:
    pool = redis.ConnectionPool.from_url(url=os.environ["REDIS_URL"])
    redis_connection = redis.Redis(connection_pool=pool)
    try:
        redis_connection.ping()
        cache = ExpiringRedisCache(redis_connection)
    except redis.ConnectionError as e:
        log.error("redis_connection_failed", exc=str(e))

log.debug("caching_requests", backend=str(cache.__class__.__name__))

CacheControl(
    requests_session,
    heuristic=ExpiresAfter(**EXPIRATION_DELTA),
    # Don't add GET here -- cache does not distinguish,
    # and GET isn't used very often in the actual checks
    cacheable_methods=["HEAD"],
    cache=cache,
)
