from .fetch import requests_session
from .issues import Issues, Issue
from .checks import registry
from . import severity


class Runner:
    def __init__(self, urls, fingerprints=[], severity=None, ignore_types=[]):
        self.urls = urls
        self.url_runners = {}
        self.fingerprints = fingerprints
        self.severity = severity
        self.ignore_types = ignore_types

    def run(self):
        for url in self.urls:
            runner = URLRunner(url)
            runner.run()
            if self.fingerprints:
                runner.issues.filter_fingerprints(self.fingerprints)
            if self.severity:
                runner.issues.filter_severity(self.severity)
            if self.ignore_types:
                runner.issues.filter_types(self.ignore_types)
            self.url_runners[url] = runner

    def as_data(self):
        data = {
            "urls": {},
            "filters": {"severity": self.severity, "fingerprints": self.fingerprints},
        }
        for url, runner in self.url_runners.items():
            data["urls"][url] = runner.as_data()
        return data


class URLRunner:
    """Runner processes a single URL"""

    def __init__(self, url):
        self.url = url
        self.content = ""
        self.status_code = None
        self.issues = Issues()

    def get_url_content(self, url):
        response = requests_session.get(url)

        self.status_code = response.status_code
        self.url = response.url
        self.content = response.text
        response.raise_for_status()

    def run(self):
        print(f"Checking {self.url}")
        try:
            self.get_url_content(self.url)
        except Exception as e:
            self.issues.append(
                Issue(
                    url=self.url,
                    type="request-failed",
                    severity=severity.ERROR,
                    content="Failed to load the url.",
                    extra={"exception": str(e)},
                )
            )
        else:
            for cls in registry.registered_checks:
                check = cls(self.url, self.content)
                check_issues = check.run()
                # all_issues = all_issues + check_issues
                self.issues._issues += check_issues._issues

    def as_data(self):
        return {
            # "url": self.url,
            "status_code": self.status_code,
            "content": self.content,
            "issues": self.issues.as_data(),
        }
